rm(list=ls())

# load packages.
library(np)

# set the seed for random number generation
seed <- 618
set.seed(seed)

h_lb <- 0.0001  # lower bound for optimal bandwidth search
h_ub <- 1  # upper bound for optimal bandwidth search

N <- 250  # sample size: 250, 500, 1000, 2000
numSims <- 5000  # simulation repetitions

beta0 <- matrix(c(1,0.5,0.5),nrow = 3,ncol = 1)  # true value for beta

# generate empty matrices to store the estimates
matB_c2 <- matrix(NaN,numSims,3)  # estimates using 2 time period cycles
matB_c3 <- matrix(NaN,numSims,3)  # estimates using 3 time period cycles

# Functions
## CM criterion function
Qn <- function(b,normIndex = 1,normVal = 1) {
  if (normIndex == 1)
    beta <- rbind(normVal,b[1],b[2])
  else if (normIndex == 2)
    beta <- rbind(b[1],normVal,b[2])
  else
    beta <- rbind(b[1],b[2],normVal)
  sum(apply(cbind(-deltaXP %*% beta,0),1,max))
}

## cross validation criterion function using 2 time period cycles
CVQn <- function(h, X, Y) {
  prob <- matrix(0, N, 6)
  bw <- array(h,dim(X)[2])
  for (j in 1:6) {
    prob[, j] <- npksum(
      X,
      Y[, j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        X,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  sum((Y-prob)^2)
}

## cross validation criterion function using 3 time period cycles
CVQn3 <- function(h, X, Y) {
  prob <- matrix(0, N, 9)
  bw <- array(h,dim(X)[2])
  for (j in 1:9) {
    prob[, j] <- npksum(
      X,
      Y[, j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        X,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  sum((Y-prob)^2)
}

# main
## DGP
for (inst in 1:numSims) {
  X <- matrix(runif(N * 3 * 3 * 2),nrow = N * 3,ncol = 3 * 2)
  omega <- matrix(runif(N * 2),nrow = N,ncol = 2)
  A <- (cbind(X[(N+1):(2*N),1]-X[1:N,1],X[(N+1):(2*N),4]-X[1:N,4]) + omega) / 4
  u01 <- matrix(rcauchy(N * 3 * 2,0,2),nrow = N * 3,ncol = 2)
  u2 <- matrix(rcauchy(N * 3,0,2),nrow = N * 3,ncol = 1)
  u <- cbind(u01,u2)
  eps <- cbind(u[,2] - u[,1],u[,3] - u[,1]) * rbind(A,A,A)
  util12 <- rbind(A,A,A) + cbind(X[,1:3] %*% beta0,X[,4:6] %*% beta0) + eps
  util <- cbind(matrix(0,N * 3,1),util12)
  choice <- max.col(util)
  Y <- matrix(0,N * 3,3)
  for (itr in 1:(N * 3)) {
    Y[itr,choice[itr]] <- 1
  }
  X <- cbind(X[1:N,],X[(N + 1):(N * 2),],X[(2*N + 1):(N * 3),])
  Y <- cbind(Y[1:N,],Y[(N + 1):(N * 2),],Y[(2*N + 1):(N * 3),])
  choice <- cbind(choice[1:N],choice[(N + 1):(N * 2)],choice[(2*N + 1):(N * 3)])
  
  
  # Cycle t1,t2
  prob <- matrix(0, N, 6)
  tmpX <- X[,1:12]
  tmpY <- Y[,1:6]
  h <- optimize(CVQn,c(h_lb,h_ub), X=tmpX, Y=tmpY)
  bw <- array(h$minimum,dim(tmpX)[2])
  for (j in 1:6) {
    prob[, j] <- npksum(
      tmpX,
      tmpY[,j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        tmpX,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  cprob12 <- prob
  deltaX12 <- X[,1:6] - X[,7:12]
  deltaP12 <- cprob12[,2:3] - cprob12[,5:6]
  deltaXP12 <- deltaX12[,1:3] * matrix(rep(deltaP12[,1],3),N,3) + deltaX12[,4:6] * matrix(rep(deltaP12[,2],3),N,3)
  
  
  # Cycle t2,t3
  prob <- matrix(0, N, 6)
  tmpX <- X[,7:18]
  tmpY <- Y[,4:9]
  h <- optimize(CVQn,c(h_lb,h_ub), X=tmpX, Y=tmpY)
  bw <- array(h$minimum,dim(tmpX)[2])
  for (j in 1:6) {
    prob[, j] <- npksum(
      tmpX,
      tmpY[,j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        tmpX,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  cprob23 <- prob
  deltaX23 <- X[,7:12] - X[,13:18]
  deltaP23 <- cprob23[,2:3] - cprob23[,5:6]
  deltaXP23 <- deltaX23[,1:3] * matrix(rep(deltaP23[,1],3),N,3) + deltaX23[,4:6] * matrix(rep(deltaP23[,2],3),N,3)
  
  # Cycle t1,t3
  prob <- matrix(0, N, 6)  
  tmpX <- X[,c(1:6,13:18)]
  tmpY <- Y[,c(1:3,7:9)]
  h <- optimize(CVQn,c(h_lb,h_ub), X=tmpX, Y=tmpY)
  bw <- array(h$minimum,dim(tmpX)[2])
  for (j in 1:6) {
    prob[, j] <- npksum(
      tmpX,
      tmpY[,j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        tmpX,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  cprob13 <- prob
  deltaX13 <- X[,1:6] - X[,13:18]
  deltaP13 <- cprob13[,2:3] - cprob13[,5:6]
  deltaXP13 <- deltaX13[,1:3] * matrix(rep(deltaP13[,1],3),N,3) + deltaX13[,4:6] * matrix(rep(deltaP13[,2],3),N,3)
  
  # do estimations using 2 time period cycles
  optObj2 <- Inf
  optBeta2 <- NaN
  ui <- matrix(c(1,0,-1,0,0,1,0,-1),4,2)
  ci <- matrix(-1,4,1)
  deltaXP <- rbind(deltaXP13,deltaXP23,deltaXP12)
  opt2 <- constrOptim(c(0, 0), Qn,NULL,ui,ci)
  optBeta2 <- rbind(1,opt2$par[1],opt2$par[2])
  matB_c2[inst,] <- optBeta2
  
  # Cycle t1,t2,t3
  prob <- matrix(0, N, 9)
  h <- optimize(CVQn3,c(h_lb,h_ub), X=X, Y=Y)
  bw <- array(h$minimum,dim(X)[2])
  for (j in 1:9) {
    prob[, j] <- npksum(
      X,
      Y[, j],
      leave.one.out = TRUE,
      bandwidth.divide = TRUE,
      bws = bw
    )$ksum /
      npksum(
        X,
        leave.one.out = TRUE,
        bandwidth.divide = TRUE,
        bws = bw
      )$ksum
  }
  cprob123 <- prob
  deltaX321 <- X[,7:12] - X[,1:6]
  deltaX332 <- X[,13:18] - X[,7:12]
  deltaX313 <- X[,1:6] - X[,13:18]
  deltaXP123 <-
    deltaX321[, 1:3] * matrix(rep(cprob123[, 2], 3), N, 3) + deltaX321[, 4:6] * matrix(rep(cprob123[, 3], 3), N, 3) +
    deltaX332[, 1:3] * matrix(rep(cprob123[, 5], 3), N, 3) + deltaX332[, 4:6] * matrix(rep(cprob123[, 6], 3), N, 3) + 
    deltaX313[, 1:3] * matrix(rep(cprob123[, 8], 3), N, 3) + deltaX313[, 4:6] * matrix(rep(cprob123[, 9], 3), N, 3)
  deltaXP123 <- -deltaXP123
  
  # do estimation using 3 time period cycles
  optObj3 <- Inf
  optBeta3 <- NaN
  deltaXP <- rbind(deltaXP13,deltaXP23,deltaXP12,deltaXP123)
  opt3 <- constrOptim(c(0, 0), Qn,NULL,ui,ci)
  optBeta3 <- rbind(1,opt3$par[1],opt3$par[2])
  matB_c3[inst,] <- optBeta3
}

# output estimates
write.table(matB_c2, file = "matB_c2", row.names = FALSE, col.names = FALSE)
write.table(matB_c3, file = "matB_c3", row.names = FALSE, col.names = FALSE)

