%----------------------------------------------------------------------
% File: estimate_10wk_15wk.m
% Description: Point estimation of the bathroom tissue demand application
% using 10 and 15 weeks.
% Last updated: 10/05/2017
%----------------------------------------------------------------------

% 10 weeks

%% Load data
clear
load data_10wk.mat;

num_week = 10;
num_store=size(store,1)/(7*num_week);
matA10=[];  % to save \deltaX*\deltaS for each store and each cycle
week_index=0:7:((num_week-1)*7);

%% Generate length 2 cycles
for i1=1:num_week
    for i2=1:num_week
        if i1<i2
            x0=week_index(i2);
            y0=week_index(i1);
            for j1=1:num_store
                matA121=zeros(1,3); % to save \deltaX*\deltaS for each store
                for j2=(j1-1)*(7*num_week)+1:(j1-1)*(7*num_week)+7  % This is k=0:K
                    matA121(1)=matA121(1)+(deal(j2+x0)-deal(j2+y0))*(mktshare(j2+x0)-mktshare(j2+y0));
                    matA121(2)=matA121(2)+(price_adj(j2+x0)-price_adj(j2+y0))*(mktshare(j2+x0)-mktshare(j2+y0));
                    matA121(3)=matA121(3)+(price_adj(j2+x0)*deal(j2+x0)-price_adj(j2+y0)*deal(j2+y0))...
                        *(mktshare(j2+x0)-mktshare(j2+y0));
                end
                matA10=[matA10;matA121];
            end
        end
    end
end

%% Optimization, search for the minimum with each possible normalization
optVal = Inf;
optBeta = [];
for normIndex = 1:3
    for normVal = 1:2
        Aeq = [0 0 0];
        Aeq(normIndex) = 1;
        beq = (-1)^normVal;
        b0 = [0;0;0];
        b0(normIndex) = (-1)^normVal;
        [beta,objVal] = fmincon(@(b)Qn(b,matA10,num_store),b0,[],[],Aeq,beq);
        if objVal < optVal
            optVal = objVal;
            optBeta = beta;
        end
    end
end

beta10 = optBeta / norm(optBeta);   % Change to unit norm normaliztion




% 15 weeks

%% Load data
load data_15wk.mat;

num_week = 15;
num_store=size(store,1)/(7*num_week);
matA15=[];  % to save \deltaX*\deltaS for each store and each cycle
week_index=0:7:((num_week-1)*7);

%% Generate length 2 cycles
for i1=1:num_week
    for i2=1:num_week
        if i1<i2
            x0=week_index(i2);
            y0=week_index(i1);            
            for j1=1:num_store
                matA121=zeros(1,3); % to save \deltaX*\deltaS for each store
                for j2=(j1-1)*(7*num_week)+1:(j1-1)*(7*num_week)+7  % This is k=0:K
                    matA121(1)=matA121(1)+(deal(j2+x0)-deal(j2+y0))*(mktshare(j2+x0)-mktshare(j2+y0));
                    matA121(2)=matA121(2)+(price_adj(j2+x0)-price_adj(j2+y0))*(mktshare(j2+x0)-mktshare(j2+y0));
                    matA121(3)=matA121(3)+(price_adj(j2+x0)*deal(j2+x0)-price_adj(j2+y0)*deal(j2+y0))...
                        *(mktshare(j2+x0)-mktshare(j2+y0));
                end
                matA15=[matA15;matA121];
            end
        end
    end
end

%% Optimization, search for the minimum with each possible normalization
optVal = Inf;
optBeta = [];
for normIndex = 1:3
    for normVal = 1:2
        Aeq = [0 0 0];
        Aeq(normIndex) = 1;
        beq = (-1)^normVal;
        b0 = [0;0;0];
        b0(normIndex) = (-1)^normVal;
        [beta,objVal] = fmincon(@(b)Qn(b,matA15,num_store),b0,[],[],Aeq,beq);
        if objVal < optVal
            optVal = objVal;
            optBeta = beta;
        end
    end
end

beta15 = optBeta / norm(optBeta);   % Change to unit norm normaliztion

beta10
beta15